import 'dart:async';

import 'package:after_layout/after_layout.dart';
import 'package:decorated_flutter/decorated_flutter.dart';
import 'package:flutter/material.dart';
import 'package:nim_fluttify/nim_fluttify.dart';
import 'package:nim_fluttify_example/src/mixins.dart';

class RecvMessageScreen extends StatefulWidget {
  @override
  _RecvMessageScreenState createState() => _RecvMessageScreenState();
}

class _RecvMessageScreenState extends State<RecvMessageScreen>
    with AccountTokenMixin, AfterLayoutMixin {
  List<MessageVM> _messageList = [];
  StreamSubscription<MessageVM> _subscription;

  @override
  void afterFirstLayout(BuildContext context) {
    _subscription = Nim.instance.messageStream.asyncMap((msg) {
      if (msg is TextMessage) {
        return TextMessageVM.data(msg);
      } else if (msg is ImageMessage) {
        return ImageMessageVM.data(msg);
      } else if (msg is VideoMessage) {
        return VideoMessageVM.data(msg);
      } else if (msg is AudioMessage) {
        return AudioMessageVM.data(msg);
      } else if (msg is FileMessage) {
        return FileMessageVM.data(msg);
      } else if (msg is CustomMessage) {
        return CustomMessageVM.data(msg);
      } else {
        return null;
      }
    }).listen((event) => setState(() => _messageList.add(event)));
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(title: Text('接收消息')),
      body: DecoratedColumn(
        children: <Widget>[
          DecoratedRow(
            itemSpacing: kSpaceNormal,
            mainAxisAlignment: MainAxisAlignment.center,
            children: <Widget>[
              Text('如果要接收聊天室消息, 请先'),
              RaisedButton(
                onPressed: handleEnterChatRoom,
                child: Text('进入聊天室'),
              ),
            ],
          ),
          ListView.builder(
            shrinkWrap: true,
            itemCount: _messageList.length,
            itemBuilder: (context, index) {
              final vm = _messageList[index];
              if (vm is TextMessageVM) {
                return _TextMessageItem(vm);
              } else if (vm is ImageMessageVM) {
                return _ImageMessageItem(vm);
              } else if (vm is VideoMessageVM) {
                return _VideoMessageItem(vm);
              } else if (vm is AudioMessageVM) {
                return _AudioMessageItem(vm);
              } else if (vm is FileMessageVM) {
                return _FileMessageItem(vm);
              } else if (vm is CustomMessageVM) {
                return _CustomMessageItem(vm);
              } else {
                return Text('未实现的类型');
              }
            },
          ),
        ],
      ),
    );
  }

  @override
  void dispose() {
    super.dispose();
    _subscription?.cancel();
  }
}

class MessageVM {
  String sessionId;
  String senderAccount;
  String senderNickname;
}

class TextMessageVM extends MessageVM {
  static Future<TextMessageVM> data(TextMessage data) async {
    return TextMessageVM()
      ..content = await data.content
      ..sessionId = await data.sessionId
      ..senderAccount = await data.senderAccount
      ..senderNickname = await data.senderNickname;
  }

  String content;
}

class FileMessageVM extends MessageVM {
  static Future<FileMessageVM> data(FileMessage data) async {
    return FileMessageVM()
      ..sessionId = await data.sessionId
      ..url = await data.url
      ..fileLength = await data.fileLength
      ..path = await data.path
      ..displayName = await data.displayName
      ..md5 = await data.md5
      ..senderAccount = await data.senderAccount
      ..senderNickname = await data.senderNickname;
  }

  String url;
  String path;
  String displayName;
  String md5;
  int fileLength;
}

class ImageMessageVM extends MessageVM {
  static Future<ImageMessageVM> data(ImageMessage data) async {
    return ImageMessageVM()
      ..sessionId = await data.sessionId
      ..imageUrl = await data.url
      ..thumbUrl = await data.thumbUrl
      ..size = await data.size
      ..senderAccount = await data.senderAccount
      ..senderNickname = await data.senderNickname;
  }

  String imageUrl;
  String thumbUrl;
  Size size;
}

class AudioMessageVM extends MessageVM {
  static Future<AudioMessageVM> data(AudioMessage data) async {
    return AudioMessageVM()
      ..sessionId = await data.sessionId
      ..url = await data.url
      ..duration = await data.duration
      ..senderAccount = await data.senderAccount
      ..senderNickname = await data.senderNickname;
  }

  String url;
  int duration;
}

class VideoMessageVM extends MessageVM {
  static Future<VideoMessageVM> data(VideoMessage data) async {
    return VideoMessageVM()
      ..sessionId = await data.sessionId
      ..url = await data.url
      ..coverUrl = await data.coverUrl
      ..duration = await data.duration
      ..senderAccount = await data.senderAccount
      ..senderNickname = await data.senderNickname;
  }

  String url;
  String coverUrl;
  int duration;
}

class CustomMessageVM extends MessageVM {
  static Future<CustomMessageVM> data(CustomMessage data) async {
    return CustomMessageVM()
      ..sessionId = await data.sessionId
      ..customString = await data.customString
      ..senderAccount = await data.senderAccount
      ..senderNickname = await data.senderNickname;
  }

  String customString;
}

class _TextMessageItem extends StatelessWidget {
  const _TextMessageItem(this.vm, {Key key}) : super(key: key);

  final TextMessageVM vm;

  @override
  Widget build(BuildContext context) {
    return Card(
      child: DecoratedColumn(
        crossAxisAlignment: CrossAxisAlignment.start,
        padding: EdgeInsets.all(kSpaceNormal),
        children: <Widget>[
          Text('文本消息', style: context.textTheme.headline4),
          Text('会话id: ${vm.sessionId}'),
          Text('发送方账户: ${vm.senderAccount}'),
          Text('发送方昵称: ${vm.senderNickname}'),
          Text('消息内容: ${vm.content}'),
        ],
      ),
    );
  }
}

class _ImageMessageItem extends StatelessWidget {
  const _ImageMessageItem(this.vm, {Key key}) : super(key: key);

  final ImageMessageVM vm;

  @override
  Widget build(BuildContext context) {
    return Card(
      child: DecoratedColumn(
        crossAxisAlignment: CrossAxisAlignment.start,
        padding: EdgeInsets.all(kSpaceNormal),
        children: <Widget>[
          Text('图片消息', style: context.textTheme.headline4),
          Text('会话id: ${vm.sessionId}'),
          Text('发送方账户: ${vm.senderAccount}'),
          Text('发送方昵称: ${vm.senderNickname}'),
          Text('图片地址: ${vm.imageUrl}'),
          Text('缩略图地址: ${vm.thumbUrl}'),
          Text('图片宽高: ${vm.size}'),
        ],
      ),
    );
  }
}

class _VideoMessageItem extends StatelessWidget {
  const _VideoMessageItem(this.vm, {Key key}) : super(key: key);

  final VideoMessageVM vm;

  @override
  Widget build(BuildContext context) {
    return Card(
      child: DecoratedColumn(
        crossAxisAlignment: CrossAxisAlignment.start,
        padding: EdgeInsets.all(kSpaceNormal),
        children: <Widget>[
          Text('视频消息', style: context.textTheme.headline4),
          Text('会话id: ${vm.sessionId}'),
          Text('发送方账户: ${vm.senderAccount}'),
          Text('发送方昵称: ${vm.senderNickname}'),
          Text('视频地址: ${vm.url}'),
          Text('缩略图地址: ${vm.coverUrl}'),
          Text('视频时长: ${vm.duration}'),
        ],
      ),
    );
  }
}

class _AudioMessageItem extends StatelessWidget {
  const _AudioMessageItem(this.vm, {Key key}) : super(key: key);

  final AudioMessageVM vm;

  @override
  Widget build(BuildContext context) {
    return Card(
      child: DecoratedColumn(
        crossAxisAlignment: CrossAxisAlignment.start,
        padding: EdgeInsets.all(kSpaceNormal),
        children: <Widget>[
          Text('音频消息', style: context.textTheme.headline4),
          Text('会话id: ${vm.sessionId}'),
          Text('发送方账户: ${vm.senderAccount}'),
          Text('发送方昵称: ${vm.senderNickname}'),
          Text('视频地址: ${vm.url}'),
          Text('视频时长: ${vm.duration}'),
        ],
      ),
    );
  }
}

class _FileMessageItem extends StatelessWidget {
  const _FileMessageItem(this.vm, {Key key}) : super(key: key);

  final FileMessageVM vm;

  @override
  Widget build(BuildContext context) {
    return Card(
      child: DecoratedColumn(
        crossAxisAlignment: CrossAxisAlignment.start,
        padding: EdgeInsets.all(kSpaceNormal),
        children: <Widget>[
          Text('文件消息', style: context.textTheme.headline4),
          Text('会话id: ${vm.sessionId}'),
          Text('发送方账户: ${vm.senderAccount}'),
          Text('发送方昵称: ${vm.senderNickname}'),
          Text('文件地址: ${vm.url}'),
          Text('文件md5: ${vm.md5}'),
          Text('文件显示名称: ${vm.displayName}'),
          Text('文件路径: ${vm.path}'),
          Text('文件大小: ${vm.fileLength}字节'),
        ],
      ),
    );
  }
}

class _CustomMessageItem extends StatelessWidget {
  const _CustomMessageItem(this.vm, {Key key}) : super(key: key);

  final CustomMessageVM vm;

  @override
  Widget build(BuildContext context) {
    return Card(
      child: DecoratedColumn(
        crossAxisAlignment: CrossAxisAlignment.start,
        padding: EdgeInsets.all(kSpaceNormal),
        children: <Widget>[
          Text('自定义消息', style: context.textTheme.headline4),
          Text('会话id: ${vm.sessionId}'),
          Text('发送方账户: ${vm.senderAccount}'),
          Text('发送方昵称: ${vm.senderNickname}'),
          Text('自定义消息: ${vm.customString}'),
        ],
      ),
    );
  }
}
