import 'dart:async';
import 'dart:convert';

import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:foundation_fluttify/foundation_fluttify.dart';
import 'package:nim_fluttify/nim_fluttify.dart';
import 'package:nim_fluttify/src/android/android.export.g.dart';
import 'package:nim_fluttify/src/ios/ios.export.g.dart';

import 'extensions.dart';

class NimUser {
  NimUser.ios(this._iosModel) : _androidModel = null;

  NimUser.android(this._androidModel) : _iosModel = null;

  final NIMUser _iosModel;
  final com_netease_nimlib_sdk_uinfo_model_NimUserInfo _androidModel;

  /// 账号
  Future<String> get account async {
    return platform(
      android: (pool) => _androidModel.getAccount(),
      ios: (pool) => _iosModel.get_userId(),
    );
  }

  /// 昵称
  Future<String> get nickname async {
    return platform(
      android: (pool) => _androidModel.getName(),
      ios: (pool) async {
        final userInfo = await _iosModel.get_userInfo();
        pool.add(userInfo);
        return userInfo.get_nickName();
      },
    );
  }

  /// 头像
  Future<String> get avatar async {
    return platform(
      android: (pool) => _androidModel.getAvatar(),
      ios: (pool) async {
        final userInfo = await _iosModel.get_userInfo();
        pool.add(userInfo);
        return userInfo.get_avatarUrl();
      },
    );
  }

  /// 生日
  Future<String> get birthday async {
    return platform(
      android: (pool) => _androidModel.getBirthday(),
      ios: (pool) async {
        final userInfo = await _iosModel.get_userInfo();
        pool.add(userInfo);
        return userInfo.get_birth();
      },
    );
  }

  /// 电子邮箱
  Future<String> get email async {
    return platform(
      android: (pool) => _androidModel.getEmail(),
      ios: (pool) async {
        final userInfo = await _iosModel.get_userInfo();
        pool.add(userInfo);
        return userInfo.get_email();
      },
    );
  }

  /// 手机号
  Future<String> get mobile async {
    return platform(
      android: (pool) => _androidModel.getMobile(),
      ios: (pool) async {
        final userInfo = await _iosModel.get_userInfo();
        pool.add(userInfo);
        return userInfo.get_mobile();
      },
    );
  }

  /// 个性签名
  Future<String> get signature async {
    return platform(
      android: (pool) => _androidModel.getSignature(),
      ios: (pool) async {
        final userInfo = await _iosModel.get_userInfo();
        pool.add(userInfo);
        return userInfo.get_sign();
      },
    );
  }

  /// 性别
  Future<Gender> get gender async {
    return platform(
      android: (pool) async {
        final gender = await _androidModel.getGenderEnum();
        switch (gender) {
          case com_netease_nimlib_sdk_uinfo_constant_GenderEnum.UNKNOWN:
            return Gender.unknown;
          case com_netease_nimlib_sdk_uinfo_constant_GenderEnum.MALE:
            return Gender.male;
          case com_netease_nimlib_sdk_uinfo_constant_GenderEnum.FEMALE:
            return Gender.female;
          default:
            return Gender.unknown;
        }
      },
      ios: (pool) async {
        final userInfo = await _iosModel.get_userInfo();
        pool.add(userInfo);
        final gender = await userInfo.get_gender();
        switch (gender) {
          case NIMUserGender.NIMUserGenderUnknown:
            return Gender.unknown;
          case NIMUserGender.NIMUserGenderMale:
            return Gender.male;
          case NIMUserGender.NIMUserGenderFemale:
            return Gender.female;
          default:
            return Gender.unknown;
        }
      },
    );
  }

  /// 扩展信息
  Future<String> get extension async {
    return platform(
      android: (pool) async => jsonEncode(await _androidModel.getExtension()),
      ios: (pool) => _iosModel.get_ext(),
    );
  }
}

class RecentSession {
  RecentSession.ios(this._iosModel) : _androidModel = null;

  RecentSession.android(this._androidModel) : _iosModel = null;

  final NIMRecentSession _iosModel;
  final com_netease_nimlib_sdk_msg_model_RecentContact _androidModel;

  Future<DateTime> get time {
    return platform(
      android: (pool) async {
        final timestamp = await _androidModel.getTime();
        return DateTime.fromMillisecondsSinceEpoch(timestamp);
      },
      ios: (pool) async {
        final timestamp = await _iosModel.get_updateTime();
        return DateTime.fromMillisecondsSinceEpoch(timestamp.ceil());
      },
    );
  }
}

/// 会话
class Session {
//  final NIMSession _iosModel;
//  final com_netease_nimlib_sdk_msg_model_ _androidModel;
}

/// 聊天室状态
class ChatRoomStatus {
  ChatRoomStatus(this.roomId, this.statusType);

  final String roomId;
  final StatusType statusType;

  @override
  String toString() {
    return 'ChatRoomStatus{roomId: $roomId, statusType: $statusType}';
  }
}

/// 踢出
class KickOut {
  KickOut(this.roomId, this.reason);

  final String roomId;
  final KickOutReason reason;

  @override
  String toString() {
    return 'KickOut{roomId: $roomId, reason: $reason}';
  }
}

/// 聊天室进入结果
class EnterChatRoomResult {
  EnterChatRoomResult.ios(this._iosChatRoom, this._iosMember)
      : _androidModel = null;

  EnterChatRoomResult.android(this._androidModel)
      : _iosChatRoom = null,
        _iosMember = null;

  final com_netease_nimlib_sdk_chatroom_model_EnterChatRoomResultData
      _androidModel;

  final NIMChatroom _iosChatRoom;
  final NIMChatroomMember _iosMember;

  Future<String> get roomId {
    return platform(
      android: (pool) => _androidModel.getRoomId(),
      ios: (pool) => _iosChatRoom.get_roomId(),
    );
  }

  Future<String> get name {
    return platform(
      android: (pool) async {
        final roomInfo = await _androidModel.getRoomInfo();
        return roomInfo.getName();
      },
      ios: (pool) => _iosChatRoom.get_name(),
    );
  }

  Future<String> get announcement {
    return platform(
      android: (pool) async {
        final roomInfo = await _androidModel.getRoomInfo();
        return roomInfo.getAnnouncement();
      },
      ios: (pool) => _iosChatRoom.get_announcement(),
    );
  }

  Future<String> get broadcastUrl {
    return platform(
      android: (pool) async {
        final roomInfo = await _androidModel.getRoomInfo();
        return roomInfo.getBroadcastUrl();
      },
      ios: (pool) => _iosChatRoom.get_broadcastUrl(),
    );
  }

  Future<String> get creator {
    return platform(
      android: (pool) async {
        final roomInfo = await _androidModel.getRoomInfo();
        return roomInfo.getCreator();
      },
      ios: (pool) => _iosChatRoom.get_creator(),
    );
  }

  Future<int> get onlineUserCount {
    return platform(
      android: (pool) async {
        final roomInfo = await _androidModel.getRoomInfo();
        return roomInfo.getOnlineUserCount();
      },
      ios: (pool) => _iosChatRoom.get_onlineUserCount(),
    );
  }
}

/// 聊天室信息
class ChatRoomInfo {
  ChatRoomInfo.ios(this._iosModel) : _androidModel = null;

  ChatRoomInfo.android(this._androidModel) : _iosModel = null;

  final com_netease_nimlib_sdk_chatroom_model_ChatRoomInfo _androidModel;
  final NIMChatroom _iosModel;

  Future<String> get roomId {
    return platform(
      android: (pool) => _androidModel.getRoomId(),
      ios: (pool) => _iosModel.get_roomId(),
    );
  }

  Future<String> get name {
    return platform(
      android: (pool) async {
        return _androidModel.getName();
      },
      ios: (pool) => _iosModel.get_name(),
    );
  }

  Future<String> get announcement {
    return platform(
      android: (pool) async {
        return _androidModel.getAnnouncement();
      },
      ios: (pool) => _iosModel.get_announcement(),
    );
  }

  Future<String> get broadcastUrl {
    return platform(
      android: (pool) async {
        return _androidModel.getBroadcastUrl();
      },
      ios: (pool) => _iosModel.get_broadcastUrl(),
    );
  }

  Future<String> get creator {
    return platform(
      android: (pool) async {
        return _androidModel.getCreator();
      },
      ios: (pool) => _iosModel.get_creator(),
    );
  }

  Future<int> get onlineUserCount {
    return platform(
      android: (pool) async {
        return _androidModel.getOnlineUserCount();
      },
      ios: (pool) => _iosModel.get_onlineUserCount(),
    );
  }
}

/// 系统消息
class SystemMessage {}

//region 消息模型类
abstract class Message {
  Message.android(this.androidModel) : this.iosModel = null;

  Message.ios(this.iosModel) : this.androidModel = null;

  final com_netease_nimlib_sdk_msg_model_IMMessage androidModel;
  final NIMMessage iosModel;

  /// 发送方账户
  Future<String> get senderAccount async {
    return platform(
      android: (pool) async {
        return androidModel.getFromAccount();
      },
      ios: (pool) async {
        return iosModel.get_from();
      },
    );
  }

  /// 发送方昵称
  Future<String> get senderNickname async {
    return platform(
      android: (pool) async {
        return androidModel.getFromNick();
      },
      ios: (pool) async {
        return iosModel.get_senderName();
      },
    );
  }

  /// 会话id
  Future<String> get sessionId async {
    return platform(
      android: (pool) async {
        return androidModel.getSessionId();
      },
      ios: (pool) async {
        return iosModel.get_session().then((value) => value.get_sessionId());
      },
    );
  }

  /// 消息时间
  Future<DateTime> get time {
    return platform(
      android: (pool) async {
        final timestamp = await androidModel.getTime();
        return DateTime.fromMillisecondsSinceEpoch(timestamp);
      },
      ios: (pool) async {
        final timestamp = await iosModel.get_timestamp();
        return DateTime.fromMillisecondsSinceEpoch((timestamp * 1000).ceil());
      },
    );
  }

  /// 会话类型
  Future<SessionType> get sessionType {
    return platform(
      android: (pool) async {
        final sessionType = await androidModel.getSessionType();
        return SessionTypeX.fromAndroid(sessionType);
      },
      ios: (pool) async {
        final session = await iosModel.get_session();
        final sessionType = await session.get_sessionType();
        return SessionTypeX.fromIOS(sessionType);
      },
    );
  }

  @protected
  Future<T> androidAttachment<
      T extends com_netease_nimlib_sdk_msg_attachment_MsgAttachment>() async {
    return TypeOpNimFluttifyAndroid(await androidModel.getAttachment())
        .as__<T>();
  }

  @protected
  Future<T> iosAttachment<T extends NIMMessageObject>() async {
    return TypeOpNimFluttifyIOS(await iosModel.get_messageObject()).as__<T>();
  }
}

/// 文字消息
class TextMessage = Message with _TextMessageMixin;

mixin _TextMessageMixin on Message {
  Future<String> get content async {
    return platform(
      android: (pool) async {
        return (await androidModel.getContent()) ?? '';
      },
      ios: (pool) async {
        return (await iosModel.get_text()) ?? '';
      },
    );
  }
}

/// 文件消息
class FileMessage = Message with _FileMessageMixin;

mixin _FileMessageMixin on Message {
  Future<String> get url async {
    return platform(
      android: (pool) async {
        final attachment = await androidAttachment<
            com_netease_nimlib_sdk_msg_attachment_FileAttachment>();
        return attachment.getUrl();
      },
      ios: (pool) async {
        final attachment = await iosAttachment<NIMFileObject>();
        return attachment.get_url();
      },
    );
  }

  Future<String> get displayName async {
    return platform(
      android: (pool) async {
        final attachment = await androidAttachment<
            com_netease_nimlib_sdk_msg_attachment_FileAttachment>();
        return attachment.getDisplayName();
      },
      ios: (pool) async {
        final attachment = await iosAttachment<NIMFileObject>();
        return attachment.get_displayName();
      },
    );
  }

  Future<String> get path async {
    return platform(
      android: (pool) async {
        final attachment = await androidAttachment<
            com_netease_nimlib_sdk_msg_attachment_FileAttachment>();
        return attachment.getPath();
      },
      ios: (pool) async {
        final attachment = await iosAttachment<NIMFileObject>();
        return attachment.get_path();
      },
    );
  }

  Future<String> get md5 async {
    return platform(
      android: (pool) async {
        final attachment = await androidAttachment<
            com_netease_nimlib_sdk_msg_attachment_FileAttachment>();
        return attachment.getMd5();
      },
      ios: (pool) async {
        final attachment = await iosAttachment<NIMFileObject>();
        return attachment.get_md5();
      },
    );
  }

  Future<int> get fileLength async {
    return platform(
      android: (pool) async {
        final attachment = await androidAttachment<
            com_netease_nimlib_sdk_msg_attachment_FileAttachment>();
        return attachment.getSize();
      },
      ios: (pool) async {
        final attachment = await iosAttachment<NIMFileObject>();
        return attachment.get_fileLength();
      },
    );
  }
}

/// 图片消息
class ImageMessage = Message with _FileMessageMixin, _ImageMessageMixin;

mixin _ImageMessageMixin on Message {
  Future<String> get thumbUrl async {
    return platform(
      android: (pool) async {
        final attachment = await androidAttachment<
            com_netease_nimlib_sdk_msg_attachment_ImageAttachment>();
        return attachment.getThumbUrl();
      },
      ios: (pool) async {
        final attachment = await iosAttachment<NIMImageObject>();
        return attachment.get_thumbUrl();
      },
    );
  }

  Future<Size> get size async {
    return platform(
      android: (pool) async {
        final attachment = await androidAttachment<
            com_netease_nimlib_sdk_msg_attachment_ImageAttachment>();
        final width = await attachment.getWidth();
        final height = await attachment.getHeight();
        return Size(width.toDouble(), height.toDouble());
      },
      ios: (pool) async {
        final attachment = await iosAttachment<NIMImageObject>();
        final size = await attachment.get_size();
        return Size(await size.width, await size.height);
      },
    );
  }
}

/// 音频消息
class AudioMessage = Message with _FileMessageMixin, _AudioMessageMixin;

mixin _AudioMessageMixin on Message {
  /// 音频时长
  Future<int> get duration async {
    return platform(
      android: (pool) async {
        final attachment = await androidAttachment<
            com_netease_nimlib_sdk_msg_attachment_AudioAttachment>();
        return attachment.getDuration();
      },
      ios: (pool) async {
        final attachment = await iosAttachment<NIMAudioObject>();
        return attachment.get_duration();
      },
    );
  }
}

/// 视频消息
class VideoMessage = Message with _FileMessageMixin, _VideoMessageMixin;

mixin _VideoMessageMixin on Message {
  /// 视频时长
  Future<int> get duration async {
    return platform(
      android: (pool) async {
        final attachment = await androidAttachment<
            com_netease_nimlib_sdk_msg_attachment_VideoAttachment>();
        return attachment.getDuration();
      },
      ios: (pool) async {
        final attachment = await iosAttachment<NIMVideoObject>();
        return attachment.get_duration();
      },
    );
  }

  /// 封面url
  Future<String> get coverUrl async {
    return platform(
      android: (pool) async {
        final attachment = await androidAttachment<
            com_netease_nimlib_sdk_msg_attachment_VideoAttachment>();
        return attachment.getThumbUrl();
      },
      ios: (pool) async {
        final attachment = await iosAttachment<NIMVideoObject>();
        return attachment.get_coverUrl();
      },
    );
  }
}

/// 自定义消息
class CustomMessage = Message with _CustomMessageMixin;

mixin _CustomMessageMixin on Message {
  /// 自定义消息
  Future<String> get customString async {
    return platform(
      android: (pool) async {
        return androidModel.getCustomAttachment();
      },
      ios: (pool) async {
        final message = await iosAttachment<NIMCustomObject>();
        return message.getAttachmentString();
      },
    );
  }
}
//endregion
