// ignore_for_file: camel_case_types
import 'dart:async';
import 'dart:io';

import 'package:flutter/cupertino.dart';
import 'package:flutter/services.dart';
import 'package:foundation_fluttify/foundation_fluttify.dart';
import 'package:nim_fluttify/src/android/android.export.g.dart';
import 'package:nim_fluttify/src/ios/ios.export.g.dart';

import 'enums.dart';
import 'extensions.dart';
import 'models.dart';

part 'callbacks.dart';

class Nim
    with
        _NimStream,
        _Auth,
        _SendMessage,
        _UserRelationship,
        _ChatRoom,
        _Session,
        _HistoryMessage {
  static Nim instance = Nim._();

  Nim._();

  /// 初始化
  Future<void> init(String appKey) async {
    await platform(
      android: (pool) async {
        await com_netease_nimlib_sdk_NIMClient.initSDK();
      },
      ios: (pool) async {
        // 创建选项参数
        final sdkOptions = await NIMSDKOption.create__();
        await sdkOptions.set_appKey(appKey);

        // 初始化
        final nim = await NIMSDK.sharedSDK();
        await nim.registerWithOption(sdkOptions);

        pool..add(sdkOptions)..add(nim);
      },
    );
    // 注册自定义消息解析类
    await MethodChannel('com.fluttify/nim_fluttify')
        .invokeMethod('attachCustomParser');
  }
}

/// 流监听
mixin _NimStream {
  final _onlineStatus = StreamController<StatusType>.broadcast();
  final _message = StreamController<Message>.broadcast();
  final _chatRoomStatus = StreamController<ChatRoomStatus>.broadcast();
  final _kickOut = StreamController<KickOut>.broadcast();
  final _systemMessage = StreamController<SystemMessage>.broadcast();

  // 是否已经设置监听, 防止多次登录导致多次设置监听
  bool _alreadySetup = false;

  /// 消息流
  Stream<Message> get messageStream => _message.stream;

  /// 在线状态流
  Stream<StatusType> get onlineStatusStream => _onlineStatus.stream;

  /// 聊天室状态流
  Stream<ChatRoomStatus> get chatRoomEventStream => _chatRoomStatus.stream;

  /// 聊天室踢出状态流
  Stream<KickOut> get kickOutStream => _kickOut.stream;

  /// 系统消息流
  Stream<SystemMessage> get systemMessageStream => _systemMessage.stream;

  Future<void> _initStream() async {
    if (!_alreadySetup) {
      _setupOnlineStatus();
      _setupOnKickOut();
      _setupChatRoomStatus();
      _setupSystemMessage();
      _setupOnMessage();

      _alreadySetup = true;
    }
  }

  /// 监听登录状态
  void _setupOnlineStatus() async {
    if (Platform.isAndroid) {
      final observer =
          await com_netease_nimlib_sdk_NIMSDK.getAuthServiceObserve();
      await observer.observeOnlineStatus(
        _AndroidCallback<com_netease_nimlib_sdk_StatusCode>(
          onEventListener: (status) => _onlineStatus.add(status.toStatusType()),
        ),
        true,
      );

      await observer.release__();
    } else if (Platform.isIOS) {
      final nim = await NIMSDK.sharedSDK();
      final loginManager = await nim.get_loginManager();
      await loginManager.addDelegate(
        _iOSCallback(
          onLoginStep: (step) => _onlineStatus.add(step.toStatusType()),
        ),
      );
      await nim.release__();
      await loginManager.release__();
    } else {
      throw '未实现的平台';
    }
  }

  /// 监听聊天室状态
  void _setupChatRoomStatus() async {
    if (Platform.isAndroid) {
      final observer = await com_netease_nimlib_sdk_NIMChatRoomSDK
          .getChatRoomServiceObserve();
      await observer.observeOnlineStatus(
        _AndroidCallback<
                com_netease_nimlib_sdk_chatroom_model_ChatRoomStatusChangeData>(
            onEventListener: (data) async {
          final roomId = await data.get_roomId();
          final status = await data.get_status();
          _chatRoomStatus.add(ChatRoomStatus(roomId, status.toStatusType()));
        }),
        true,
      );

      await observer.release__();
    } else if (Platform.isIOS) {
      final nim = await NIMSDK.sharedSDK();
      final manager = await nim.get_chatroomManager();
      await manager.addDelegate(
        _iOSCallback(onChatRoomStatus: _chatRoomStatus.add),
      );
      await nim.release__();
      await manager.release__();
    } else {
      throw '未实现的平台';
    }
  }

  /// 监听消息
  void _setupOnMessage() async {
    if (Platform.isAndroid) {
      final msgObserver =
          await com_netease_nimlib_sdk_NIMSDK.getMsgServiceObserve();
      await msgObserver.observeReceiveMessage(
        _AndroidCallback<List<com_netease_nimlib_sdk_msg_model_IMMessage>>(
          onEventListener: (messageList) {
            messageList.forEach((element) async {
              _message.add(await element.toMessage());
            });
          },
        ),
        true,
      );
      final chatroomObserver = await com_netease_nimlib_sdk_NIMChatRoomSDK
          .getChatRoomServiceObserve();
      await chatroomObserver.observeReceiveMessage(
        _AndroidCallback<
            List<com_netease_nimlib_sdk_chatroom_model_ChatRoomMessage>>(
          onEventListener: (messageList) {
            messageList.forEach((element) async {
              _message.add(await element.toMessage());
            });
          },
        ),
        true,
      );
      await msgObserver.release__();
      await chatroomObserver.release__();
    } else if (Platform.isIOS) {
      final nim = await NIMSDK.sharedSDK();
      final manager = await nim.get_chatManager();
      await manager.addDelegate(
        _iOSCallback(
          onMessageList: (messageList) {
            messageList.forEach((element) async {
              _message.add(await element.toMessage());
            });
          },
        ),
      );
      await nim.release__();
      await manager.release__();
    } else {
      throw '未实现的平台';
    }
  }

  /// 监听踢出聊天室事件
  void _setupOnKickOut() async {
    if (Platform.isAndroid) {
      final observer = await com_netease_nimlib_sdk_NIMChatRoomSDK
          .getChatRoomServiceObserve();
      await observer.observeKickOutEvent(
        _AndroidCallback<
                com_netease_nimlib_sdk_chatroom_model_ChatRoomKickOutEvent>(
            onEventListener: (data) async {
          final roomId = await data.getRoomId();
          final reason = await data.getReason();
          _kickOut.add(KickOut(roomId, reason.toKickOutReason()));
        }),
        true,
      );

      await observer.release__();
    } else if (Platform.isIOS) {
      final nim = await NIMSDK.sharedSDK();
      final manager = await nim.get_chatroomManager();
      await manager.addDelegate(_iOSCallback(onKickOut: _kickOut.add));

      await nim.release__();
      await manager.release__();
    } else {
      throw '未实现的平台';
    }
  }

  /// 监听系统消息
  void _setupSystemMessage() async {
//    if (Platform.isAndroid) {
//      final observer =
//          await com_netease_nimlib_sdk_NIMSDK_X.getSystemMessageObserver();
//      await observer.observeReceiveSystemMsg(
//        _AndroidCallback<com_netease_nimlib_sdk_msg_model_SystemMessage>(
//            onEventListener: (data) async {
//          final roomId = await data.getRoomId();
//          final reason = await data.getReason();
//          _kickOut.add(KickOut(roomId, reason.toKickOutReason()));
//        }),
//        true,
//      );
//
//      await observer.release__();
//    } else if (Platform.isIOS) {
//      final nim = await NIMSDK.sharedSDK();
//      final manager = await nim.get_systemNotificationManager();
//      await manager.addDelegate(_iOSCallback(onKickOut: _kickOut.add));
//
//      await nim.release__();
//      await manager.release__();
//    } else {
//      throw '未实现的平台';
//    }
  }

  /// 释放资源
  void dispose() {
    _onlineStatus.close();
    _message.close();
    _chatRoomStatus.close();
    _kickOut.close();
    _systemMessage.close();
  }
}

/// 登录登出
mixin _Auth on _NimStream {
  /// 登录
  ///
  /// [account] 用户账户
  /// [token] 登录密码
  /// sdk会自动连接服务器，传递用户信息，返回登录结果。
  /// 该操作中途可取消。如果因为网络比较差，或其他原因导致服务器迟迟没有返回，用户也没有主动取消，
  /// 在45秒后AbortableFuture的onFailed会被调用到。
  Future<void> login({@required String account, @required String token}) async {
    final completer = Completer<void>();
    await platform(
      android: (pool) async {
        // 获取service
        final service = await com_netease_nimlib_sdk_NIMSDK.getAuthService();

        // 创建登录参数
        final loginInfo = await com_netease_nimlib_sdk_auth_LoginInfo
            .create__String__String(account, token);

        // 执行登录
        final future = await service.login(loginInfo);
        await future.setCallback(
          _AndroidCallback<com_netease_nimlib_sdk_auth_LoginInfo>(
            onSuccessListener: (loginInfo) {
              completer.complete();
            },
            onFailedListener: (error) {
              completer.completeError(error);
            },
          ),
        );

        pool..add(service)..add(loginInfo);

        return completer.future;
      },
      ios: (pool) async {
        // 获取服务
        final nim = await NIMSDK.sharedSDK();
        final loginManager = await nim.get_loginManager();

        // 执行登录
        await loginManager.login_token_completion(
          account,
          token,
          (error) async {
            if (error == null) {
              completer.complete();
            } else {
              completer.completeError(await error.code);
            }
          },
        );

        pool..add(nim)..add(loginManager);
        return completer.future;
      },
    );
    // 登录完成后, 初始化一下监听流
    await _initStream();
  }

  /// 登出
  ///
  /// 如果用户手动登出，不再接收消息和提醒，开发者可以调用 logout 方法，该方法没有回调。
  Future<void> logout() async {
    return platform(
      android: (pool) async {
        // 获取服务
        final service = await com_netease_nimlib_sdk_NIMSDK.getAuthService();
        // 执行登出
        await service.logout();

        pool..add(service);
      },
      ios: (pool) async {
        // 获取服务
        final nim = await NIMSDK.sharedSDK();
        final loginManager = await nim.get_loginManager();
        // 执行登出
        await loginManager.logout((error) {});

        pool..add(nim)..add(loginManager);
      },
    );
  }
}

/// 发送消息
mixin _SendMessage {
  /// 发送文字消息
  ///
  /// [sessionId] 会话id, 单聊时为对方account, 群聊/聊天室为房间id
  /// [type] 会话类型, 目前分为单聊, 群聊和聊天室
  /// [content] 发送的文字内容
  Future<void> sendTextMessage({
    @required String sessionId,
    @required SessionType type,
    @required String content,
  }) async {
    final completer = Completer<void>();
    return platform(
      android: (pool) async {
        // 聊天室
        if (type == SessionType.chatroom) {
          final message =
              await com_netease_nimlib_sdk_chatroom_ChatRoomMessageBuilder
                  .createChatRoomTextMessage(sessionId, content);

          // 获取服务
          final service =
              await com_netease_nimlib_sdk_NIMChatRoomSDK.getChatRoomService();
          // 发送消息
          final future = await service.sendMessage(message, false);
          // 设置回调
          await future.setCallback(_AndroidCallback(
            onSuccessListener: (_) => completer.complete(),
            onFailedListener: completer.completeError,
          ));
          pool..add(message)..add(service);
        } else {
          // 创建消息对象
          final message = await com_netease_nimlib_sdk_msg_MessageBuilder
              .createTextMessage(sessionId, type.androidModel(), content);

          // 获取服务
          final service = await com_netease_nimlib_sdk_NIMSDK.getMsgService();
          // 发送消息
          final future = await service.sendMessage(message, false);
          // 回调
          await future.setCallback(_AndroidCallback(
            onSuccessListener: (_) => completer.complete(),
            onFailedListener: completer.completeError,
          ));

          pool..add(message)..add(service);
        }
        return completer.future;
      },
      ios: (pool) async {
        // 获取服务
        final nim = await NIMSDK.sharedSDK();
        final manager = await nim.get_chatManager();

        // 创建消息对象
        final session =
            await NIMSession.session_type(sessionId, type.iosModel());
        final message = await NIMMessage.create__();
        await message.set_text(content);

        pool..add(nim)..add(manager)..add(session)..add(message);
        // 发送消息
        return manager.sendMessage_toSession_completion(
          message,
          session,
          (error) async {
            if (error == null) {
              completer.complete();
            } else {
              completer.completeError(await error.code);
            }
          },
        );
      },
    );
  }

  /// 发送图片消息
  ///
  /// [sessionId] 会话id, 单聊时为对方account, 群聊/聊天室为房间id
  /// [type] 会话类型, 目前分为单聊, 群聊和聊天室
  /// [imageFile] 发送的图片文件
  Future<void> sendImageMessage({
    @required String sessionId,
    @required SessionType type,
    @required File imageFile,
  }) async {
    final completer = Completer<void>();
    return platform(
      android: (pool) async {
        // 创建消息对象
        final file = await java_io_File.create(imageFile.path);
        // 聊天室
        if (type == SessionType.chatroom) {
          final message =
              await com_netease_nimlib_sdk_chatroom_ChatRoomMessageBuilder
                  .createChatRoomImageMessage__String__java_io_File__String(
            sessionId,
            file,
            '' /* displayName */,
          );

          // 获取服务
          final service =
              await com_netease_nimlib_sdk_NIMChatRoomSDK.getChatRoomService();
          // 发送消息
          final future = await service.sendMessage(message, false);
          // 设置回调
          await future.setCallback(_AndroidCallback(
            onSuccessListener: (_) => completer.complete(),
            onFailedListener: completer.completeError,
          ));
          pool..add(message)..add(service);
        }
        // 私聊/群聊
        else {
          final message = await com_netease_nimlib_sdk_msg_MessageBuilder
              .createImageMessage__String__com_netease_nimlib_sdk_msg_constant_SessionTypeEnum__java_io_File(
            sessionId,
            type.androidModel(),
            file,
          );

          // 获取服务
          final service = await com_netease_nimlib_sdk_NIMSDK.getMsgService();
          // 发送消息
          final future = await service.sendMessage(message, false);
          // 设置回调
          await future.setCallback(_AndroidCallback(
            onSuccessListener: (_) => completer.complete(),
            onFailedListener: completer.completeError,
          ));
          pool..add(message)..add(service);
        }

        return completer.future;
      },
      ios: (pool) async {
        // 获取服务
        final nim = await NIMSDK.sharedSDK();
        final manager = await nim.get_chatManager();

        // 创建消息对象
        final session =
            await NIMSession.session_type(sessionId, type.iosModel());

        final messageObject = await NIMImageObject.create__();
        await messageObject.initWithFilepath(imageFile.path);

        final message = await NIMMessage.create__();
        await message.set_messageObject(messageObject);

        pool..add(nim)..add(manager)..add(session)..add(message);
        // 发送消息
        return manager.sendMessage_toSession_completion(
          message,
          session,
          (error) async {
            if (error == null) {
              completer.complete();
            } else {
              completer.completeError(await error.code);
            }
          },
        );
      },
    );
  }

  /// 发送音频消息
  ///
  /// [sessionId] 会话id, 单聊时为对方account, 群聊/聊天室为房间id
  /// [type] 会话类型, 目前分为单聊, 群聊和聊天室
  /// [audioFile] 发送的音频文件
  /// [duration] 音频时长
  Future<void> sendAudioMessage({
    @required String sessionId,
    @required SessionType type,
    @required File audioFile,
    @required Duration duration,
  }) async {
    final completer = Completer<void>();
    return platform(
      android: (pool) async {
        // 创建消息对象
        final file = await java_io_File.create(audioFile.path);
        // 聊天室
        if (type == SessionType.chatroom) {
          final message =
              await com_netease_nimlib_sdk_chatroom_ChatRoomMessageBuilder
                  .createChatRoomAudioMessage__String__java_io_File__int(
            sessionId,
            file,
            duration.inMilliseconds,
          );

          // 获取服务
          final service =
              await com_netease_nimlib_sdk_NIMChatRoomSDK.getChatRoomService();
          // 发送消息
          final future = await service.sendMessage(message, false);
          // 设置回调
          await future.setCallback(_AndroidCallback(
            onSuccessListener: (_) => completer.complete(),
            onFailedListener: completer.completeError,
          ));
          pool..add(message)..add(service);
        }
        // 私聊/群聊
        else {
          final message = await com_netease_nimlib_sdk_msg_MessageBuilder
              .createAudioMessage__String__com_netease_nimlib_sdk_msg_constant_SessionTypeEnum__java_io_File__int(
            sessionId,
            type.androidModel(),
            file,
            duration.inMilliseconds,
          );

          // 获取服务
          final service = await com_netease_nimlib_sdk_NIMSDK.getMsgService();
          // 发送消息
          final future = await service.sendMessage(message, false);

          await future.setCallback(_AndroidCallback(
            onSuccessListener: (_) => completer.complete(),
            onFailedListener: completer.completeError,
          ));

          pool..add(message)..add(service);
        }
        return completer.future;
      },
      ios: (pool) async {
        // 获取服务
        final nim = await NIMSDK.sharedSDK();
        final manager = await nim.get_chatManager();

        // 创建消息对象
        final session =
            await NIMSession.session_type(sessionId, type.iosModel());

        final messageObject = await NIMAudioObject.create__();
        await messageObject.initWithSourcePath(audioFile.path);
        await messageObject.set_duration(duration.inMilliseconds);

        final message = await NIMMessage.create__();
        await message.set_messageObject(messageObject);

        pool..add(nim)..add(manager)..add(session)..add(message);
        // 发送消息
        return manager.sendMessage_toSession_completion(
          message,
          session,
          (error) async {
            if (error == null) {
              completer.complete();
            } else {
              completer.completeError(await error.code);
            }
          },
        );
      },
    );
  }

  /// 发送视频消息
  ///
  /// [sessionId]   聊天对象ID
  /// [sessionType] 会话类型
  /// [file]        视频文件对象
  /// [duration]    视频文件持续时间
  /// [width]       视频宽度
  /// [height]      视频高度
  /// [displayName] 视频文件显示名，可以为空
  Future<void> sendVideoMessage({
    @required String sessionId,
    @required SessionType type,
    @required File videoFile,
    @required Duration duration,
    @required int width,
    @required int height,
    String displayName,
  }) async {
    final completer = Completer<void>();
    return platform(
      android: (pool) async {
        // 创建消息对象
        final file = await java_io_File.create(videoFile.path);
        // 聊天室
        if (type == SessionType.chatroom) {
          final message =
              await com_netease_nimlib_sdk_chatroom_ChatRoomMessageBuilder
                  .createChatRoomVideoMessage__String__java_io_File__int__int__int__String(
            sessionId,
            file,
            duration.inMilliseconds,
            width,
            height,
            displayName,
          );

          // 获取服务
          final service =
              await com_netease_nimlib_sdk_NIMChatRoomSDK.getChatRoomService();
          // 发送消息
          final future = await service.sendMessage(message, false);
          // 设置回调
          await future.setCallback(_AndroidCallback(
            onSuccessListener: (_) => completer.complete(),
            onFailedListener: completer.completeError,
          ));
          pool..add(message)..add(service);
        }
        // 私聊/群聊
        else {
          final message = await com_netease_nimlib_sdk_msg_MessageBuilder
              .createVideoMessage__String__com_netease_nimlib_sdk_msg_constant_SessionTypeEnum__java_io_File__int__int__int__String(
            sessionId,
            type.androidModel(),
            file,
            duration.inMilliseconds,
            width,
            height,
            displayName,
          );

          // 获取服务
          final service = await com_netease_nimlib_sdk_NIMSDK.getMsgService();
          // 发送消息
          final future = await service.sendMessage(message, false);

          await future.setCallback(_AndroidCallback(
            onSuccessListener: (_) => completer.complete(),
            onFailedListener: completer.completeError,
          ));

          pool..add(message)..add(service);
        }
        return completer.future;
      },
      ios: (pool) async {
        // 获取服务
        final nim = await NIMSDK.sharedSDK();
        final manager = await nim.get_chatManager();

        // 创建消息对象
        final session =
            await NIMSession.session_type(sessionId, type.iosModel());

        final messageObject = await NIMVideoObject.create__();
        await messageObject.initWithSourcePath(videoFile.path);
        await messageObject.set_duration(duration.inMilliseconds);

        final message = await NIMMessage.create__();
        await message.set_messageObject(messageObject);

        pool..add(nim)..add(manager)..add(session)..add(message);
        // 发送消息
        return manager.sendMessage_toSession_completion(
          message,
          session,
          (error) async {
            if (error == null) {
              completer.complete();
            } else {
              completer.completeError(await error.code);
            }
          },
        );
      },
    );
  }

  /// 发送文件消息
  ///
  /// [sessionId]   聊天对象ID
  /// [sessionType] 会话类型
  /// [file]        文件对象
  /// [displayName] 文件显示名，可以为空
  Future<void> sendFileMessage({
    @required String sessionId,
    @required SessionType type,
    @required File file,
    String displayName,
  }) async {
    final completer = Completer<void>();
    return platform(
      android: (pool) async {
        // 创建消息对象
        final _file = await java_io_File.create(file.path);
        // 聊天室
        if (type == SessionType.chatroom) {
          final message =
              await com_netease_nimlib_sdk_chatroom_ChatRoomMessageBuilder
                  .createChatRoomFileMessage__String__java_io_File__String(
            sessionId,
            _file,
            displayName ?? '',
          );

          // 获取服务
          final service =
              await com_netease_nimlib_sdk_NIMChatRoomSDK.getChatRoomService();
          // 发送消息
          final future = await service.sendMessage(message, false);
          // 设置回调
          await future.setCallback(_AndroidCallback(
            onSuccessListener: (_) => completer.complete(),
            onFailedListener: completer.completeError,
          ));
          pool..add(message)..add(service);
        }
        // 私聊/群聊
        else {
          final message = await com_netease_nimlib_sdk_msg_MessageBuilder
              .createFileMessage__String__com_netease_nimlib_sdk_msg_constant_SessionTypeEnum__java_io_File__String(
            sessionId,
            type.androidModel(),
            _file,
            displayName ?? '',
          );

          // 获取服务
          final service = await com_netease_nimlib_sdk_NIMSDK.getMsgService();
          // 发送消息
          final future = await service.sendMessage(message, false);

          await future.setCallback(_AndroidCallback(
            onSuccessListener: (_) => completer.complete(),
            onFailedListener: completer.completeError,
          ));

          pool..add(message)..add(service);
        }
        return completer.future;
      },
      ios: (pool) async {
        // 获取服务
        final nim = await NIMSDK.sharedSDK();
        final manager = await nim.get_chatManager();

        // 创建消息对象
        final session =
            await NIMSession.session_type(sessionId, type.iosModel());

        final messageObject = await NIMFileObject.create__();
        await messageObject.initWithSourcePath(file.path);
        await messageObject.set_displayName(displayName ?? '');

        final message = await NIMMessage.create__();
        await message.set_messageObject(messageObject);

        pool..add(nim)..add(manager)..add(session)..add(message);
        // 发送消息
        return manager.sendMessage_toSession_completion(
          message,
          session,
          (error) async {
            if (error == null) {
              completer.complete();
            } else {
              completer.completeError(await error.code);
            }
          },
        );
      },
    );
  }

  /// 发送位置消息
  ///
  /// [sessionId] 会话id, 单聊时为对方account, 群聊/聊天室为房间id
  /// [type] 会话类型, 目前分为单聊, 群聊和聊天室
  /// [latitude] 纬度
  /// [longitude] 经度
  /// [address] 地址
  Future<void> sendLocationMessage({
    @required String sessionId,
    @required SessionType type,
    @required double latitude,
    @required double longitude,
    @required String address,
  }) async {
    final completer = Completer<void>();
    return platform(
      android: (pool) async {
        // 聊天室
        if (type == SessionType.chatroom) {
          final message =
              await com_netease_nimlib_sdk_chatroom_ChatRoomMessageBuilder
                  .createChatRoomLocationMessage(
            sessionId,
            latitude,
            longitude,
            address,
          );

          // 获取服务
          final service =
              await com_netease_nimlib_sdk_NIMChatRoomSDK.getChatRoomService();
          // 发送消息
          final future = await service.sendMessage(message, false);
          // 设置回调
          await future.setCallback(_AndroidCallback(
            onSuccessListener: (_) => completer.complete(),
            onFailedListener: completer.completeError,
          ));
          pool..add(message)..add(service);
        }
        // 私聊/群聊
        else {
          // 创建消息对象
          final message = await com_netease_nimlib_sdk_msg_MessageBuilder
              .createLocationMessage(
            sessionId,
            type.androidModel(),
            latitude,
            longitude,
            address,
          );

          // 获取服务
          final service = await com_netease_nimlib_sdk_NIMSDK.getMsgService();
          // 发送消息
          final future = await service.sendMessage(message, false);

          await future.setCallback(_AndroidCallback(
            onSuccessListener: (_) => completer.complete(),
            onFailedListener: completer.completeError,
          ));

          pool..add(message)..add(service);
        }
        return completer.future;
      },
      ios: (pool) async {
        // 获取服务
        final nim = await NIMSDK.sharedSDK();
        final manager = await nim.get_chatManager();

        // 创建消息对象
        final session =
            await NIMSession.session_type(sessionId, type.iosModel());

        final messageObject = await NIMLocationObject.create__();
        await messageObject.initWithLatitude_longitude_title(
          latitude,
          longitude,
          address,
        );

        final message = await NIMMessage.create__();
        await message.set_messageObject(messageObject);

        pool..add(nim)..add(manager)..add(session)..add(message);
        // 发送消息
        return manager.sendMessage_toSession_completion(
          message,
          session,
          (error) async {
            if (error == null) {
              completer.complete();
            } else {
              completer.completeError(await error.code);
            }
          },
        );
      },
    );
  }

  /// 发送自定义消息
  ///
  /// [sessionId]   聊天对象ID
  /// [sessionType] 会话类型
  /// [attachmentString] 自定义消息体
  Future<void> sendCustomMessage({
    @required String sessionId,
    @required SessionType type,
    @required String attachmentString,
  }) async {
    return platform(
      android: (pool) async {
        final completer = Completer<void>();
        // 聊天室
        if (type == SessionType.chatroom) {
          final message =
              await com_netease_nimlib_sdk_chatroom_ChatRoomMessageBuilder_X
                  .createChatRoomCustomMessage(
            sessionId,
            attachmentString,
          );

          // 获取服务
          final service =
              await com_netease_nimlib_sdk_NIMChatRoomSDK.getChatRoomService();
          // 发送消息
          final future = await service.sendMessage(message, false);
          // 设置回调
          await future.setCallback(_AndroidCallback(
            onSuccessListener: (_) => completer.complete(),
            onFailedListener: completer.completeError,
          ));
          pool..add(message)..add(service);
        }
        // 私聊/群聊
        else {
          // 创建消息对象
          final message = await com_netease_nimlib_sdk_msg_MessageBuilder_X
              .createCustomMessage(
            sessionId,
            type,
            attachmentString,
          );

          // 获取服务
          final service = await com_netease_nimlib_sdk_NIMSDK.getMsgService();
          // 发送消息
          final future = await service.sendMessage(message, false);

          await future.setCallback(_AndroidCallback(
            onSuccessListener: (_) => completer.complete(),
            onFailedListener: completer.completeError,
          ));

          pool..add(message)..add(service);
        }
        return completer.future;
      },
      ios: (pool) async {
        final completer = Completer<void>();
        // 获取服务
        final nim = await NIMSDK.sharedSDK();
        final manager = await nim.get_chatManager();

        // 创建消息对象
        final session =
            await NIMSession.session_type(sessionId, type.iosModel());

        final messageObject = await NIMCustomObject.create__();
        await messageObject.setAttachmentString(attachmentString);

        final message = await NIMMessage.create__();
        await message.set_messageObject(messageObject);

        pool..add(nim)..add(manager)..add(session)..add(message);
        // 发送消息
        return manager.sendMessage_toSession_completion(
          message,
          session,
          (error) async {
            if (error == null) {
              completer.complete();
            } else {
              completer.completeError(await error.code);
            }
          },
        );
      },
    );
  }
}

/// 用户关系
mixin _UserRelationship {
  /// 获取我的好友列表
  Future<List<NimUser>> fetchMyFriendList() async {
    return platform(
      android: (pool) async {
        // 获取服务
        final friendService =
            await com_netease_nimlib_sdk_NIMSDK.getFriendService();
        final userService =
            await com_netease_nimlib_sdk_NIMSDK.getUserService();
        // 获取好友列表
        final friends = await friendService.getFriendAccounts();
        final friendInfoList = await userService.getUserInfoList(friends);

        pool..add(friendService)..add(userService);
        return <NimUser>[
          for (final item in friendInfoList) NimUser.android(item)
        ];
      },
      ios: (pool) async {
        // 获取服务
        final nim = await NIMSDK.sharedSDK();
        final manager = await nim.get_userManager();
        // 获取好友列表
        final friends = await manager.myFriends();

        pool..add(nim)..add(manager);
        return <NimUser>[for (final item in friends) NimUser.ios(item)];
      },
    );
  }

  /// 获取免打扰列表
  Future<List<NimUser>> fetchMuteList() async {
    return platform(
      android: (pool) async {
        // 获取服务
        final friendService =
            await com_netease_nimlib_sdk_NIMSDK.getFriendService();
        final userService =
            await com_netease_nimlib_sdk_NIMSDK.getUserService();
        // 获取好友列表
        final mute = await friendService.getMuteList();
        final muteList = await userService.getUserInfoList(mute);

        pool..add(friendService)..add(userService);
        return <NimUser>[for (final item in muteList) NimUser.android(item)];
      },
      ios: (pool) async {
        // 获取服务
        final nim = await NIMSDK.sharedSDK();
        final manager = await nim.get_userManager();
        // 获取好友列表
        final muteList = await manager.myMuteUserList();

        pool..add(nim)..add(manager);
        return <NimUser>[for (final item in muteList) NimUser.ios(item)];
      },
    );
  }

  /// 请求添加好友
  ///
  /// [account] 对方账号
  /// [verifyType] 验证类型, 分为`直接添加`和`需要验证`
  /// [message] 请求信息
  Future<void> requestAddFriend({
    @required String account,
    @required P2PVerifyType verifyType,
    String message = '',
  }) async {
    final completer = Completer<void>();
    return platform(
      android: (pool) async {
        // 获取服务
        final service = await com_netease_nimlib_sdk_NIMSDK.getFriendService();

        // 获取验证类型
        com_netease_nimlib_sdk_friend_constant_VerifyType _type;
        switch (verifyType) {
          case P2PVerifyType.direct:
            _type =
                com_netease_nimlib_sdk_friend_constant_VerifyType.DIRECT_ADD;
            break;
          case P2PVerifyType.request:
            _type = com_netease_nimlib_sdk_friend_constant_VerifyType
                .VERIFY_REQUEST;
            break;
        }

        // 创建申请对象
        final data = await com_netease_nimlib_sdk_friend_model_AddFriendData
            .create__String__com_netease_nimlib_sdk_friend_constant_VerifyType__String(
          account,
          _type,
          message,
        );
        // 发起申请
        final future = await service.addFriend(data);
        await future.setCallback(_AndroidCallback(
          onSuccessListener: (_) => completer.complete(),
          onFailedListener: completer.completeError,
        ));

        pool..add(service)..add(data);
        return completer.future;
      },
      ios: (pool) async {
        // 获取服务
        final nim = await NIMSDK.sharedSDK();
        final manager = await nim.get_userManager();

        // 获取验证类型
        NIMUserOperation _type;
        switch (verifyType) {
          case P2PVerifyType.direct:
            _type = NIMUserOperation.NIMUserOperationAdd;
            break;
          case P2PVerifyType.request:
            _type = NIMUserOperation.NIMUserOperationRequest;
            break;
        }

        // 创建申请对象
        final request = await NIMUserRequest.create__();
        await request.set_userId(account);
        await request.set_operation(_type);
        await request.set_message(message);

        // 发起申请
        await manager.requestFriend_completion(
          request,
          (error) async {
            if (error == null) {
              completer.complete();
            } else {
              completer.completeError(await error.code);
            }
          },
        );

        pool..add(nim)..add(manager)..add(request);
        return completer.future;
      },
    );
  }

  /// 删除好友
  ///
  /// [account] 对方账户
  /// [removeAlias] 是否同时删除别名
  Future<void> deleteFriend(String account, {bool removeAlias = false}) async {
    return platform(
      android: (pool) async {
        final completer = Completer<void>();
        // 获取服务
        final service = await com_netease_nimlib_sdk_NIMSDK.getFriendService();

        // 发起删除
        final future = await service.deleteFriend__String__bool(
          account,
          removeAlias,
        );
        await future.setCallback(_AndroidCallback(
          onSuccessListener: (_) => completer.complete(),
          onFailedListener: completer.completeError,
        ));

        pool..add(service);
        return completer.future;
      },
      ios: (pool) async {
        final completer = Completer<void>();
        // 获取服务
        final nim = await NIMSDK.sharedSDK();
        final manager = await nim.get_userManager();

        // 发起删除
        await manager.deleteFriend_removeAlias_completion(
          account,
          removeAlias,
          (error) async {
            if (error == null) {
              completer.complete();
            } else {
              completer.completeError(await error.code);
            }
          },
        );

        pool..add(nim)..add(manager);
        return completer.future;
      },
    );
  }

  /// 处理添加好友申请(同意/拒绝)
  ///
  /// [account] 对方账户
  /// [agree] 是否同意
  Future<void> ackAddFriendRequest(
    String account, {
    @required bool agree,
  }) async {
    return platform(
      android: (pool) async {
        final completer = Completer<void>();
        // 获取服务
        final service = await com_netease_nimlib_sdk_NIMSDK.getFriendService();

        // 处理
        final future = await service.ackAddFriendRequest(
          account,
          agree,
        );
        await future.setCallback(_AndroidCallback(
          onSuccessListener: (_) => completer.complete(),
          onFailedListener: completer.completeError,
        ));

        pool..add(service);
        return completer.future;
      },
      ios: (pool) async {
        final completer = Completer<void>();
        // 获取服务
        final nim = await NIMSDK.sharedSDK();
        final manager = await nim.get_userManager();

        final request = await NIMUserRequest.create__();
        await request.set_userId(account);
        await request.set_operation(agree
            ? NIMUserOperation.NIMUserOperationVerify
            : NIMUserOperation.NIMUserOperationReject);

        // 处理请求
        await manager.requestFriend_completion(
          request,
          (error) async {
            if (error == null) {
              completer.complete();
            } else {
              completer.completeError(await error.code);
            }
          },
        );

        pool..add(nim)..add(manager);
        return completer.future;
      },
    );
  }

  /// 判断用户是好友
  Future<bool> isMyFriend(String account) async {
    return platform(
      android: (pool) async {
        // 获取服务
        final service = await com_netease_nimlib_sdk_NIMSDK.getFriendService();

        pool..add(service);
        return service.isMyFriend(account);
      },
      ios: (pool) async {
        // 获取服务
        final nim = await NIMSDK.sharedSDK();
        final manager = await nim.get_userManager();

        pool..add(nim)..add(manager);
        return manager.isMyFriend(account);
      },
    );
  }

  /// 加入黑名单
  Future<void> addToBlackList(String account) async {
    final completer = Completer<void>();
    return platform(
      android: (pool) async {
        // 获取服务
        final service = await com_netease_nimlib_sdk_NIMSDK.getFriendService();

        // 添加黑名单
        final future = await service.addToBlackList(account);
        await future.setCallback(_AndroidCallback(
          onSuccessListener: (_) => completer.complete(),
          onFailedListener: completer.completeError,
        ));

        pool..add(service);
        return completer.future;
      },
      ios: (pool) async {
        // 获取服务
        final nim = await NIMSDK.sharedSDK();
        final manager = await nim.get_userManager();

        // 添加黑名单
        await manager.addToBlackList_completion(
          account,
          (error) async {
            if (error == null) {
              completer.complete();
            } else {
              completer.completeError(await error.code);
            }
          },
        );

        pool..add(nim)..add(manager);
        return completer.future;
      },
    );
  }

  /// 移出黑名单
  Future<void> removeFromBlackList(String account) async {
    return platform(
      android: (pool) async {
        final completer = Completer<void>();
        // 获取服务
        final service = await com_netease_nimlib_sdk_NIMSDK.getFriendService();

        // 移除黑名单
        final future = await service.removeFromBlackList(account);
        await future.setCallback(_AndroidCallback(
          onSuccessListener: (_) => completer.complete(),
          onFailedListener: completer.completeError,
        ));

        pool..add(service);
        return completer.future;
      },
      ios: (pool) async {
        final completer = Completer<void>();
        // 获取服务
        final nim = await NIMSDK.sharedSDK();
        final manager = await nim.get_userManager();

        // 移除黑名单
        await manager.removeFromBlackBlackList_completion(
          account,
          (error) async {
            if (error == null) {
              completer.complete();
            } else {
              completer.completeError(await error.code);
            }
          },
        );

        pool..add(nim)..add(manager);
        return completer.future;
      },
    );
  }

  /// 判断用户是否在黑名单
  Future<bool> isInBlackList(String account) async {
    return platform(
      android: (pool) async {
        // 获取服务
        final service = await com_netease_nimlib_sdk_NIMSDK.getFriendService();

        pool..add(service);
        return service.isInBlackList(account);
      },
      ios: (pool) async {
        // 获取服务
        final nim = await NIMSDK.sharedSDK();
        final manager = await nim.get_userManager();

        pool..add(nim)..add(manager);
        return manager.isUserInBlackList(account);
      },
    );
  }

  /// 判断用户是免打扰
  Future<bool> isMute(String account) async {
    return platform(
      android: (pool) async {
        // 获取服务
        final service = await com_netease_nimlib_sdk_NIMSDK.getFriendService();

        pool..add(service);
        return !await service.isNeedMessageNotify(account);
      },
      ios: (pool) async {
        // 获取服务
        final nim = await NIMSDK.sharedSDK();
        final manager = await nim.get_userManager();

        pool..add(nim)..add(manager);
        return !await manager.notifyForNewMsg(account);
      },
    );
  }

  /// 获取黑名单列表
  Future<List<NimUser>> fetchBlackList() async {
    return platform(
      android: (pool) async {
        // 获取服务
        final friendService =
            await com_netease_nimlib_sdk_NIMSDK.getFriendService();
        final userService =
            await com_netease_nimlib_sdk_NIMSDK.getUserService();
        // 获取黑名单列表
        final blackList = await friendService.getBlackList();

        pool..add(friendService)..add(userService);
        if (blackList.isEmpty) {
          return [];
        } else {
          final blackInfoList = await userService.getUserInfoList(blackList);
          return <NimUser>[
            for (final item in blackInfoList) NimUser.android(item)
          ];
        }
      },
      ios: (pool) async {
        // 获取服务
        final nim = await NIMSDK.sharedSDK();
        final manager = await nim.get_userManager();
        // 获取黑名单列表
        final blackList = await manager.myBlackList();

        pool..add(nim)..add(manager);
        return <NimUser>[for (final item in blackList) NimUser.ios(item)];
      },
    );
  }

  /// 是否接收用户[account]的消息
  ///
  /// [mute] true为免打扰(静音)
  Future<void> setMute(String account, {@required bool mute}) async {
    return platform(
      android: (pool) async {
        final completer = Completer<void>();
        // 获取服务
        final service = await com_netease_nimlib_sdk_NIMSDK.getFriendService();

        final future = await service.setMessageNotify(account, !mute);
        await future.setCallback(_AndroidCallback(
          onSuccessListener: (_) => completer.complete(),
          onFailedListener: completer.completeError,
        ));
        pool..add(service);
      },
      ios: (pool) async {
        final completer = Completer<void>();
        // 获取服务
        final nim = await NIMSDK.sharedSDK();
        final manager = await nim.get_userManager();

        pool..add(nim)..add(manager);
        await manager.updateNotifyState_forUser_completion(
          !mute,
          account,
          (error) async {
            if (error == null) {
              completer.complete();
            } else {
              completer.completeError(await error.code);
            }
          },
        );
      },
    );
  }
}

/// 聊天室
mixin _ChatRoom {
  /// 进入聊天室
  ///
  /// [chatRoomId] 聊天室id
  /// [retryCount] 重试次数
  Future<EnterChatRoomResult> enterChatRoom(
    String chatRoomId, {
    int retryCount = 3,
  }) async {
    return platform(
      android: (pool) async {
        final completer = Completer<EnterChatRoomResult>();
        // 获取服务
        final service =
            await com_netease_nimlib_sdk_NIMChatRoomSDK.getChatRoomService();

        // 创建聊天室申请对象
        final request =
            await com_netease_nimlib_sdk_chatroom_model_EnterChatRoomData
                .create__String(chatRoomId);

        // 进入聊天室
        final future = await service.enterChatRoomEx(request, retryCount);
        await future.setCallback(_AndroidCallback(
          onSuccessListener: (result) {
            completer.complete(EnterChatRoomResult.android(result));
          },
          onFailedListener: completer.completeError,
        ));

        pool..add(service);
        return completer.future;
      },
      ios: (pool) async {
        final completer = Completer<EnterChatRoomResult>();
        // 获取服务
        final nim = await NIMSDK.sharedSDK();
        final manager = await nim.get_chatroomManager();

        // 创建聊天室申请对象
        final request = await NIMChatroomEnterRequest.create__();
        await request.set_roomId(chatRoomId);
        await request.set_retryCount(retryCount);

        // 进入聊天室
        await manager.enterChatroom_completion(
          request,
          (error, chatRoom, me) async {
            if (error == null) {
              completer.complete(EnterChatRoomResult.ios(chatRoom, me));
            } else {
              completer.completeError(await error.code);
            }
          },
        );

        pool..add(nim)..add(manager)..add(request);
        return completer.future;
      },
    );
  }

  /// 退出聊天室
  Future<void> exitChatRoom(String chatRoomId) async {
    return platform(
      android: (pool) async {
        // 获取服务
        final service =
            await com_netease_nimlib_sdk_NIMChatRoomSDK.getChatRoomService();

        // 退出聊天室
        await service.exitChatRoom(chatRoomId);

        pool..add(service);
      },
      ios: (pool) async {
        // 获取服务
        final nim = await NIMSDK.sharedSDK();
        final manager = await nim.get_chatroomManager();

        // 退出聊天室
        await manager.exitChatroom_completion(
          chatRoomId,
          (error) async {
            if (error != null) {
              throw await error.code;
            }
          },
        );

        pool..add(nim)..add(manager);
      },
    );
  }

  /// 获取聊天室信息
  Future<ChatRoomInfo> fetchChatRoomInfo(String chatRoomId) async {
    return platform(
      android: (pool) async {
        final completer = Completer<ChatRoomInfo>();

        // 获取服务
        final service =
            await com_netease_nimlib_sdk_NIMChatRoomSDK.getChatRoomService();

        // 获取聊天室信息
        final future = await service.fetchRoomInfo(chatRoomId);
        await future.setCallback(_AndroidCallback(
          onSuccessListener: (result) {
            completer.complete(ChatRoomInfo.android(result));
          },
          onFailedListener: completer.completeError,
        ));

        pool..add(service);
        return completer.future;
      },
      ios: (pool) async {
        final completer = Completer<ChatRoomInfo>();

        // 获取服务
        final nim = await NIMSDK.sharedSDK();
        final manager = await nim.get_chatroomManager();

        // 退出聊天室
        await manager.fetchChatroomInfo_completion(
          chatRoomId,
          (error, chatRoom) async {
            if (error == null) {
              completer.complete(ChatRoomInfo.ios(chatRoom));
            } else {
              completer.completeError(await error.code);
            }
          },
        );

        pool..add(nim)..add(manager);
        return completer.future;
      },
    );
  }
}

/// 会话
mixin _Session {
  /// 获取会话列表
  Future<List<RecentSession>> fetchSessionList() async {
    return platform(
      android: (pool) async {
        // 获取服务
        final service = await com_netease_nimlib_sdk_NIMSDK.getMsgService();
        // 获取最近回话列表
        final sessionList = await service.queryRecentContactsBlock();

        pool..add(service);
        return <RecentSession>[
          for (final item in sessionList) RecentSession.android(item)
        ];
      },
      ios: (pool) async {
        // 获取服务
        final nim = await NIMSDK.sharedSDK();
        final manager = await nim.get_conversationManager();
        // 获取最近回话列表
        final sessionList = await manager.allRecentSessions();

        pool..add(nim)..add(manager);
        return <RecentSession>[
          for (final item in sessionList) RecentSession.ios(item)
        ];
      },
    );
  }
}

/// 历史消息
mixin _HistoryMessage {
  /// 获取消息列表
  ///
  /// [sessionId] 会话id, 单聊时为对方account, 群聊/聊天室为房间id
  /// [sessionType] 会话类型, 目前分为单聊, 群聊和聊天室
  /// [anchor] 以此消息为锚点获取下一页的消息列表
  /// [limit] 消息分页大小
  Future<List<Message>> fetchMessageList(
    String sessionId, {
    @required SessionType sessionType,
    Message anchor,
    int limit = 20,
  }) async {
    return platform(
      android: (pool) async {
        final completer = Completer<List<Message>>();

        // 获取服务
        final service = await com_netease_nimlib_sdk_NIMSDK.getMsgService();

        final com_netease_nimlib_sdk_msg_model_IMMessage _anchor = anchor
                ?.androidModel ??
            await com_netease_nimlib_sdk_msg_MessageBuilder.createEmptyMessage(
              sessionId,
              sessionType.androidModel(),
              0, // 查询起点
            );

        final messageList = await service.queryMessageListExX(
          _anchor,
          com_netease_nimlib_sdk_msg_model_QueryDirectionEnum.QUERY_NEW,
          limit,
          false, // 查询结果的排序规则 asc
        );
        completer.complete(<Message>[
          for (final item in messageList) await item.toMessage(),
        ]);

        pool..add(service)..add(_anchor);
        return completer.future;
      },
      ios: (pool) async {
        // 获取服务
        final nim = await NIMSDK.sharedSDK();
        final manager = await nim.get_conversationManager();
        // 创建session
        final session =
            await NIMSession.session_type(sessionId, sessionType.iosModel());
        // 获取消息列表
        final messageList = await manager.messagesInSession_message_limit(
          session,
          anchor?.iosModel,
          limit,
        );

        pool..add(nim)..add(manager)..add(session);
        return <Message>[
          for (final item in messageList) await item.toMessage()
        ];
      },
    );
  }
}
